package evenosmaster.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.net.UnknownHostException;
import java.nio.charset.MalformedInputException;
import java.nio.file.Files;
import java.nio.file.Path;

import org.apache.commons.compress.utils.Charsets;

import com.mongodb.BasicDBObject;
import com.mongodb.CommandResult;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.MongoClient;
import com.mongodb.MongoClientOptions;
import com.mongodb.MongoException;
import com.mongodb.WriteConcern;
import com.mongodb.WriteResult;
import com.mongodb.util.JSON;
import com.mongodb.util.JSONParseException;

public class MongoDBLoader {
	private static final String shardingType = "hashed";
	private static final String shardKeyField = "revision.id";
	private static final BasicDBObject shardKey = new BasicDBObject(shardKeyField, shardingType);
	protected final MongoClient mongoClient;
	protected final String databaseName;
	
	private StatisticsCollector stats;

	public MongoDBLoader(String host, int port, String databaseName, StatisticsCollector stats)
			throws UnknownHostException {
		this.databaseName = databaseName;
		mongoClient = createMongoClient(host, port);

		this.stats = stats;

		enableShardingOfDatabase();
		enableShardingOfCollection("nodlosning");
	}

	private static MongoClient createMongoClient(String host, int port) throws UnknownHostException {
		MongoClientOptions options = MongoClientOptions.builder().connectionsPerHost(1000).build();
		MongoClient mongoClient = new MongoClient(host + ":" + port, options);
		
		return mongoClient;
	}

	public void enableShardingOfDatabase() {
		CommandResult result = mongoClient.getDB("admin").command(new BasicDBObject("enablesharding", databaseName));
		if (!result.ok()) {
			System.out.println("Enabeling sharding for database: \"" + databaseName + "\" returned: " + result);
		}
	}

	public void insertCollectionFromFile(Path file) throws IOException {
		DB db = mongoClient.getDB(databaseName);

		BufferedReader br = Files.newBufferedReader(file, Charsets.ISO_8859_1);
		String collectionName = null;
		try {
			// The first line of the file is the collection name
			collectionName = br.readLine().trim();
		} catch (MalformedInputException e) {
			System.err.println(file);
			e.printStackTrace();
			return;
		}

//		DBCollection collection = db.getCollection(collectionName);
		DBCollection collection = db.getCollection("masterCollection");

		System.out.println(Thread.currentThread().getName() + " : " + collection.getName() + " - "
				+ file.toFile().length());
//		enableShardingOfCollection(collectionName);

		// Each consecutive line contains a JSON document
		String json = null;
		try {
			json = br.readLine();
		} catch (MalformedInputException e) {
			System.err.println(file);
			e.printStackTrace();
			json = ""; // line is erroneous, the remaining might be fine.
		}

		while (json != null) {
			json = json.trim();
			if (!json.isEmpty()) {
				inseertJSON(collection, json);
			}

			try {
				json = br.readLine();
			} catch (MalformedInputException e) {
				System.err.println(file);
				e.printStackTrace();
				// Even tho this line failed, the remaining
				// lines might still be valid
				json = "";
				continue;
			}
		}

		// System.out.println(collectionName + ": " + count + " -DONE!");

		br.close();
	}

	private void inseertJSON(DBCollection collection, String json) {
		BasicDBObject obj = null;
		try {
			obj = (BasicDBObject) JSON.parse(json);
		} catch (JSONParseException e) {
			System.err.println("Exeption inserting into collection: " + collection.getName());
			System.out.println(e.getStackTrace());
			return;
		}

		try {
			WriteResult wr = collection.insert(obj, WriteConcern.ERRORS_IGNORED);
			if (wr.getError() != null) {
				System.err.println(wr);
//				return;
			}

			// Update statistics
			stats.incrementDocumentCount();
			stats.increaseAccumolatedDocumentSize(json.length());

		} catch (MongoException e) {
			System.err.println("==== " + e.getMessage());
		}

	}

	private void enableShardingOfCollection(final String collectionName) {
		BasicDBObject cmd = new BasicDBObject("shardcollection", databaseName + "." + collectionName);
		cmd.put("key", shardKey);
		
		System.out.println(cmd.toString());

		CommandResult result = mongoClient.getDB("admin").command(cmd);

		if (!result.ok()) {
			// In most cases this is due to sharding already being enabled.
			System.out.println("Enabeling sharding for collection: \"" + collectionName + "\" returned: " + result);
		}
	}
}
